@@ -3,7 +3,7 @@ import os
 def configuration(parent_package='',top_path=None):
     from numpy.distutils.misc_util import Configuration
     config = Configuration('external', parent_package, top_path)
-    config.add_data_files('d3.v3.min.js')
+    config.add_data_files('d3.js')
     return config
 
 if __name__ == '__main__':
