@@ -24,6 +24,7 @@ from textwrap import wrap
 from datetime import datetime as dt
 from dateutil.parser import parse as parseutc
 from warnings import warn
+from nipype.external import six
 
 
 from .traits_extension import (traits, Undefined, TraitDictObject,
@@ -37,6 +38,7 @@ from ..utils.misc import is_container, trim, str2bool
 from ..utils.provenance import write_provenance
 from .. import config, logging, LooseVersion
 from .. import __version__
+import random, time, fnmatch
 
 nipype_version = LooseVersion(__version__)
 
@@ -45,6 +47,25 @@ iflogger = logging.getLogger('interface')
 
 __docformat__ = 'restructuredtext'
 
+def _lock_files():
+    tmpdir = '/tmp'
+    pattern = '.X*-lock'
+    names = fnmatch.filter(os.listdir(tmpdir), pattern)
+    ls = [os.path.join(tmpdir, child) for child in names]
+    ls = [p for p in ls if os.path.isfile(p)]
+    return ls
+
+def _search_for_free_display():
+    ls = [int(x.split('X')[1].split('-')[0]) for x in _lock_files()]
+    min_display_num = 1000
+    if len(ls):
+        display_num = max(min_display_num, max(ls) + 1)
+    else:
+        display_num = min_display_num
+    random.seed()
+    display_num += random.randint(0, 100)
+    return display_num
+
 
 def load_template(name):
     """Load a template from the script_templates directory
@@ -541,7 +562,7 @@ class BaseTraitedSpec(traits.HasTraits):
                 out = tuple(out)
         else:
             if isdefined(object):
-                if (hash_files and isinstance(object, str) and
+                if (hash_files and isinstance(object, six.string_types) and
                         os.path.isfile(object)):
                     if hash_method is None:
                         hash_method = config.get('execution', 'hash_method')
@@ -700,6 +721,7 @@ class BaseInterface(Interface):
     input_spec = BaseInterfaceInputSpec
     _version = None
     _additional_metadata = []
+    _redirect_x = False
 
     def __init__(self, **inputs):
         if not self.input_spec:
@@ -731,6 +753,7 @@ class BaseInterface(Interface):
         desc = spec.desc
         xor = spec.xor
         requires = spec.requires
+        argstr = spec.argstr
 
         manhelpstr = ['\t%s' % name]
 
@@ -753,6 +776,17 @@ class BaseInterface(Interface):
                                    initial_indent='\t\t',
                                    subsequent_indent='\t\t')
 
+        if argstr:
+            pos = spec.position
+            if pos is not None:
+                manhelpstr += wrap('flag: %s, position: %s' % (argstr, pos), 70,
+                                   initial_indent='\t\t',
+                                   subsequent_indent='\t\t')
+            else:
+                manhelpstr += wrap('flag: %s' % argstr, 70,
+                                   initial_indent='\t\t',
+                                   subsequent_indent='\t\t')
+
         if xor:
             line = '%s' % ', '.join(xor)
             manhelpstr += wrap(line, 70,
@@ -785,7 +819,7 @@ class BaseInterface(Interface):
 
         opthelpstr = ['', '\t[Optional]']
         for name, spec in sorted(inputs.traits(transient=None).items()):
-            if spec in mandatory_items:
+            if name in mandatory_items:
                 continue
             opthelpstr += cls._get_trait_desc(inputs, name, spec)
 
@@ -943,7 +977,30 @@ class BaseInterface(Interface):
                         hostname=getfqdn(),
                         version=self.version)
         try:
+            if self._redirect_x:
+                exist_val, _ = self._exists_in_path('Xvfb',
+                                                    runtime.environ)
+                if not exist_val:
+                    raise IOError("Xvfb could not be found on host %s" %
+                                  (runtime.hostname))
+                else:
+                    vdisplay_num = _search_for_free_display()
+                    xvfb_cmd = ['Xvfb', ':%d' % vdisplay_num]
+                    xvfb_proc = subprocess.Popen(xvfb_cmd,
+                                                 stdout=open(os.devnull),
+                                                 stderr=open(os.devnull))
+                    time.sleep(0.2)  # give Xvfb time to start
+                    if xvfb_proc.poll() is not None:
+                        raise Exception('Error: Xvfb did not start')
+                    old_displaynum = os.environ['DISPLAY']
+                    runtime.environ['DISPLAY'] = ':%s' % vdisplay_num
+
             runtime = self._run_interface(runtime)
+
+            if self._redirect_x:
+                xvfb_proc.kill()
+                xvfb_proc.wait()
+
             outputs = self.aggregate_outputs(runtime)
             runtime.endTime = dt.isoformat(dt.utcnow())
             timediff = parseutc(runtime.endTime) - parseutc(runtime.startTime)
@@ -972,7 +1029,7 @@ class BaseInterface(Interface):
             else:
                 inputs_str = ''
 
-            if len(e.args) == 1 and isinstance(e.args[0], str):
+            if len(e.args) == 1 and isinstance(e.args[0], six.string_types):
                 e.args = (e.args[0] + " ".join([message, inputs_str]),)
             else:
                 e.args += (message, )
@@ -1109,12 +1166,26 @@ def run_command(runtime, output=None, timeout=0.01):
     The returned runtime contains a merged stdout+stderr log with timestamps
     """
     PIPE = subprocess.PIPE
-    proc = subprocess.Popen(runtime.cmdline,
-                             stdout=PIPE,
-                             stderr=PIPE,
-                             shell=True,
-                             cwd=runtime.cwd,
-                             env=runtime.environ)
+
+    if output == 'file':
+        errfile = os.path.join(runtime.cwd, 'stderr.nipype')
+        outfile = os.path.join(runtime.cwd, 'stdout.nipype')
+        stderr = open(errfile, 'wt')
+        stdout = open(outfile, 'wt')
+
+        proc = subprocess.Popen(runtime.cmdline,
+                                stdout=stdout,
+                                stderr=stderr,
+                                shell=True,
+                                cwd=runtime.cwd,
+                                env=runtime.environ)
+    else:
+        proc = subprocess.Popen(runtime.cmdline,
+                                 stdout=PIPE,
+                                 stderr=PIPE,
+                                 shell=True,
+                                 cwd=runtime.cwd,
+                                 env=runtime.environ)
     result = {}
     errfile = os.path.join(runtime.cwd, 'stderr.nipype')
     outfile = os.path.join(runtime.cwd, 'stdout.nipype')
@@ -1154,14 +1225,6 @@ def run_command(runtime, output=None, timeout=0.01):
         result['stderr'] = stderr.split('\n')
         result['merged'] = ''
     if output == 'file':
-        stderr = open(errfile, 'wt')
-        stdout = open(outfile, 'wt')
-        proc = subprocess.Popen(runtime.cmdline,
-                                stdout=stdout,
-                                stderr=stderr,
-                                shell=True,
-                                cwd=runtime.cwd,
-                                env=runtime.environ)
         ret_code = proc.wait()
         stderr.flush()
         stdout.flush()
@@ -1325,11 +1388,12 @@ class CommandLine(BaseInterface):
 
     def _get_environ(self):
         out_environ = {}
-        try:
-            display_var = config.get('execution', 'display_variable')
-            out_environ = {'DISPLAY': display_var}
-        except NoOptionError:
-            pass
+        if not self._redirect_x:
+            try:
+                display_var = config.get('execution', 'display_variable')
+                out_environ = {'DISPLAY': display_var}
+            except NoOptionError:
+                pass
         iflogger.debug(out_environ)
         if isdefined(self.inputs.environ):
             out_environ.update(self.inputs.environ)
@@ -1469,18 +1533,22 @@ class CommandLine(BaseInterface):
             source = getattr(self.inputs, name_source)
             while isinstance(source, list):
                 source = source[0]
-            _, base, _ = split_filename(source)
+            #special treatment for files
+            try:
+                _, base, _ = split_filename(source)
+            except AttributeError:
+                base = source
             retval = name_template % base
             _, _, ext = split_filename(retval)
             if trait_spec.keep_extension and ext:
                 return retval
-            return self._overload_extension(retval)
+            return self._overload_extension(retval, name)
         return retval
 
     def _gen_filename(self, name):
         raise NotImplementedError
 
-    def _overload_extension(self, value):
+    def _overload_extension(self, value, name=None):
         return value
 
     def _list_outputs(self):
