@@ -5,10 +5,12 @@
 
 from .connectivity import Conmat
 from .convert import (Image2Voxel, FSL2Scheme, VtkStreamlines, ProcStreamlines,
-                      TractShredder, DT2NIfTI, NIfTIDT2Camino, AnalyzeHeader)
+                      TractShredder, DT2NIfTI, NIfTIDT2Camino, AnalyzeHeader,
+                      Shredder)
 from .dti import (DTIFit, ModelFit, DTLUTGen, PicoPDFs, Track, TrackPICo,
                   TrackBayesDirac, TrackDT, TrackBallStick, TrackBootstrap,
                   ComputeFractionalAnisotropy, ComputeMeanDiffusivity,
                   ComputeTensorTrace,  ComputeEigensystem, DTMetric)
 from .calib import (SFPICOCalibData, SFLUTGen)
-from .odf import (QBallMX, LinRecon, SFPeaks)
+from .odf import (QBallMX, LinRecon, SFPeaks, MESD)
+from .utils import ImageStats
