@@ -16,7 +16,8 @@ import os
 import shutil
 import warnings
 
-from nipype.interfaces.fsl.base import FSLCommand, FSLCommandInputSpec, Info
+from ... import LooseVersion
+from nipype.interfaces.fsl.base import (FSLCommand, FSLCommandInputSpec, Info)
 from nipype.interfaces.base import (TraitedSpec, isdefined, File, Directory,
                                     InputMultiPath, OutputMultiPath, traits)
 from nipype.utils.filemanip import fname_presuffix, split_filename, copyfile
@@ -26,9 +27,8 @@ warnings.filterwarnings('always', category=UserWarning)
 
 
 class DTIFitInputSpec(FSLCommandInputSpec):
-
     dwi = File(exists=True, desc='diffusion weighted image data file',
-                  argstr='-k %s', position=0, mandatory=True)
+               argstr='-k %s', position=0, mandatory=True)
     base_name = traits.Str("dtifit_", desc='base_name that all output files will start with',
                            argstr='-o %s', position=1, usedefault=True)
     mask = File(exists=True, desc='bet binary mask file',
@@ -46,13 +46,12 @@ class DTIFitInputSpec(FSLCommandInputSpec):
     save_tensor = traits.Bool(desc='save the elements of the tensor',
                         argstr='--save_tensor')
     sse = traits.Bool(desc='output sum of squared errors', argstr='--sse')
-    cni = File(exists=True, desc='input counfound regressors', argstr='-cni %s')
+    cni = File(exists=True, desc='input counfound regressors', argstr='--cni=%s')
     little_bit = traits.Bool(desc='only process small area of brain',
                              argstr='--littlebit')
 
 
 class DTIFitOutputSpec(TraitedSpec):
-
     V1 = File(exists=True, desc='path/name of file with the 1st eigenvector')
     V2 = File(exists=True, desc='path/name of file with the 2nd eigenvector')
     V3 = File(exists=True, desc='path/name of file with the 3rd eigenvector')
@@ -100,24 +99,411 @@ class DTIFit(FSLCommand):
         return outputs
 
 
-class BEDPOSTXInputSpec(FSLCommandInputSpec):
-    dwi = File(exists=True, desc='diffusion weighted image data file', mandatory=True)
+class FSLXCommandInputSpec(FSLCommandInputSpec):
+    dwi = File(exists=True, argstr='--data=%s', mandatory=True,
+               desc='diffusion weighted image data file')
+    mask = File(exists=True, argstr='--mask=%s', mandatory=True,
+                desc='brain binary mask file (i.e. from BET)')
+    bvecs = File(exists=True, argstr='--bvecs=%s', mandatory=True,
+                 desc='b vectors file')
+    bvals = File(exists=True, argstr='--bvals=%s', mandatory=True,
+                 desc='b values file')
+
+    logdir = Directory('.', argstr='--logdir=%s', usedefault=True)
+    n_fibres = traits.Range(low=1, argstr='--nfibres=%d', desc=('Maximum '
+                            'number of fibres to fit in each voxel'))
+    model = traits.Enum(1, 2, argstr='--model=%d',
+                        desc=('use monoexponential (1, default, required for '
+                              'single-shell) or multiexponential (2, multi-'
+                              'shell) model'))
+    fudge = traits.Int(argstr='--fudge=%d',
+                       desc='ARD fudge factor')
+    n_jumps = traits.Int(5000, argstr='--njumps=%d',
+                         desc='Num of jumps to be made by MCMC')
+    burn_in = traits.Range(low=0, default=0, argstr='--burnin=%d',
+                           desc=('Total num of jumps at start of MCMC to be '
+                                 'discarded'))
+    burn_in_no_ard = traits.Range(low=0, default=0, argstr='--burninnoard=%d',
+                                  desc=('num of burnin jumps before the ard is'
+                                        ' imposed'))
+    sample_every = traits.Range(low=0, default=1, argstr='--sampleevery=%d',
+                                desc='Num of jumps for each sample (MCMC)')
+    update_proposal_every = traits.Range(low=1, default=40,
+                                         argstr='--updateproposalevery=%d',
+                                         desc=('Num of jumps for each update '
+                                               'to the proposal density std '
+                                               '(MCMC)'))
+    seed = traits.Int(argstr='--seed=%d',
+                      desc='seed for pseudo random number generator')
+
+    _xor_inputs1 = ('no_ard', 'all_ard')
+    no_ard = traits.Bool(argstr='--noard', xor=_xor_inputs1,
+                         desc='Turn ARD off on all fibres')
+    all_ard = traits.Bool(argstr='--allard', xor=_xor_inputs1,
+                          desc='Turn ARD on on all fibres')
+
+    _xor_inputs2 = ('no_spat', 'non_linear', 'cnlinear')
+    no_spat = traits.Bool(argstr='--nospat', xor=_xor_inputs2,
+                          desc='Initialise with tensor, not spatially')
+    non_linear = traits.Bool(argstr='--nonlinear', xor=_xor_inputs2,
+                             desc='Initialise with nonlinear fitting')
+    cnlinear = traits.Bool(argstr='--cnonlinear', xor=_xor_inputs2,
+                           desc=('Initialise with constrained nonlinear '
+                                 'fitting'))
+    rician = traits.Bool(argstr='--rician', desc=('use Rician noise modeling'))
+
+    _xor_inputs3 = ['f0_noard', 'f0_ard']
+    f0_noard = traits.Bool(argstr='--f0', xor=_xor_inputs3,
+                           desc=('Noise floor model: add to the model an '
+                                 'unattenuated signal compartment f0'))
+    f0_ard = traits.Bool(argstr='--f0 --ardf0', xor=_xor_inputs3 + ['all_ard'],
+                         desc=('Noise floor model: add to the model an '
+                               'unattenuated signal compartment f0'))
+    force_dir = traits.Bool(True, argstr='--forcedir', usedefault=True,
+                            desc=('use the actual directory name given '
+                                  '(do not add + to make a new directory)'))
+
+
+class FSLXCommandOutputSpec(TraitedSpec):
+    dsamples = File(desc=('Samples from the distribution on diffusivity d'))
+    d_stdsamples = File(desc=('Std of samples from the distribution d'))
+    dyads = OutputMultiPath(File(), desc=('Mean of PDD distribution'
+                                          ' in vector form.'))
+    fsamples = OutputMultiPath(File(), desc=('Samples from the '
+                               'distribution on f anisotropy'))
+    mean_dsamples = File(desc='Mean of distribution on diffusivity d')
+    mean_d_stdsamples = File(desc='Mean of distribution on diffusivity d')
+    mean_fsamples = OutputMultiPath(File(), desc=('Mean of '
+                                    'distribution on f anisotropy'))
+    mean_S0samples = File(desc=('Mean of distribution on T2w'
+                                'baseline signal intensity S0'))
+    mean_tausamples = File(desc=('Mean of distribution on '
+                                 'tau samples (only with rician noise)'))
+    phsamples = OutputMultiPath(File(), desc=('phi samples, per fiber'))
+    thsamples = OutputMultiPath(File(), desc=('theta samples, per fiber'))
+
+
+class FSLXCommand(FSLCommand):
+    """
+    Base support for ``xfibres`` and ``bedpostx``
+    """
+    input_spec = FSLXCommandInputSpec
+    output_spec = FSLXCommandOutputSpec
+
+    def _run_interface(self, runtime):
+        self._out_dir = os.getcwd()
+        runtime = super(FSLXCommand, self)._run_interface(runtime)
+        if runtime.stderr:
+            self.raise_exception(runtime)
+        return runtime
+
+    def _list_outputs(self):
+        outputs = self.output_spec().get()
+        out_dir = self._out_dir
+
+        if isdefined(self.inputs.logdir):
+            out_dir = os.path.abspath(self.inputs.logdir)
+        else:
+            out_dir = os.path.abspath('logdir')
+
+        multi_out = ['dyads', 'fsamples', 'mean_fsamples',
+                     'phsamples', 'thsamples']
+        single_out = ['dsamples', 'd_stdsamples', 'mean_dsamples',
+                      'mean_S0samples', 'mean_d_stdsamples']
+
+        for k in single_out:
+            outputs[k] = self._gen_fname(k, cwd=out_dir)
+
+        if isdefined(self.inputs.rician) and self.inputs.rician:
+            outputs['mean_tausamples'] = self._gen_fname('mean_tausamples',
+                                                         cwd=out_dir)
+
+        for k in multi_out:
+            outputs[k] = []
+
+        for i in xrange(self.inputs.n_fibres + 1):
+            outputs['fsamples'].append(self._gen_fname('f%dsamples' % i,
+                                       cwd=out_dir))
+            outputs['mean_fsamples'].append(self._gen_fname(('mean_f%d'
+                                            'samples') % i, cwd=out_dir))
+
+        for i in xrange(1, self.inputs.n_fibres + 1):
+            outputs['dyads'].append(self._gen_fname('dyads%d' % i,
+                                    cwd=out_dir))
+            outputs['phsamples'].append(self._gen_fname('ph%dsamples' % i,
+                                        cwd=out_dir))
+            outputs['thsamples'].append(self._gen_fname('th%dsamples' % i,
+                                        cwd=out_dir))
+
+        return outputs
+
+
+class BEDPOSTX5InputSpec(FSLXCommandInputSpec):
+    out_dir = Directory('.', mandatory=True, desc='output directory',
+                        usedefault=True, position=1, argstr='%s')
+    gradnonlin = traits.Bool(False, argstr='-g', desc=('consider gradient '
+                             'nonlinearities, default off'))
+    use_gpu = traits.Bool(False, desc='Use the GPU version of bedpostx')
+
+
+class BEDPOSTX5OutputSpec(FSLXCommandOutputSpec):
+    mean_thsamples = OutputMultiPath(File(), desc=('Mean of '
+                                     'distribution on theta'))
+    mean_phsamples = OutputMultiPath(File(), desc=('Mean of '
+                                     'distribution on phi'))
+
+    merged_thsamples = OutputMultiPath(File(), desc=('Samples from '
+                                       'the distribution on theta'))
+    merged_phsamples = OutputMultiPath(File(), desc=('Samples from '
+                                       'the distribution on phi'))
+    merged_fsamples = OutputMultiPath(File(),
+                                      desc=('Samples from the distribution on '
+                                            'anisotropic volume fraction.'))
+    dyads_disp = OutputMultiPath(File(), desc=('Uncertainty on the '
+                                 ' estimated fiber orientation'))
+
+
+class BEDPOSTX5(FSLXCommand):
+    """
+    BEDPOSTX stands for Bayesian Estimation of Diffusion Parameters Obtained
+    using Sampling Techniques. The X stands for modelling Crossing Fibres.
+    bedpostx runs Markov Chain Monte Carlo sampling to build up distributions
+    on diffusion parameters at each voxel. It creates all the files necessary
+    for running probabilistic tractography. For an overview of the modelling
+    carried out within bedpostx see this `technical report
+    <http://www.fmrib.ox.ac.uk/analysis/techrep/tr03tb1/tr03tb1/index.html>`_.
+
+
+    .. note:: Consider using
+      :func:`nipype.workflows.fsl.dmri.create_bedpostx_pipeline` instead.
+
+
+    Example
+    -------
+
+    >>> from nipype.interfaces import fsl
+    >>> bedp = fsl.BEDPOSTX5(bvecs='bvecs', bvals='bvals', dwi='diffusion.nii',
+    ...                     mask='mask.nii', n_fibres=1)
+    >>> bedp.cmdline
+    'bedpostx . --bvals=bvals --bvecs=bvecs --data=diffusion.nii \
+--forcedir --logdir=. --mask=mask.nii --nfibres=1'
+
+    """
+
+    _cmd = 'bedpostx'
+    _default_cmd = _cmd
+    input_spec = BEDPOSTX5InputSpec
+    output_spec = BEDPOSTX5OutputSpec
+    _can_resume = True
+
+    def __init__(self, **inputs):
+        super(BEDPOSTX5, self).__init__(**inputs)
+        self.inputs.on_trait_change(self._cuda_update, 'use_gpu')
+
+    def _cuda_update(self):
+        if isdefined(self.inputs.use_gpu) and self.inputs.use_gpu:
+            self._cmd = 'bedpostx_gpu'
+        else:
+            self._cmd = self._default_cmd
+
+    def _run_interface(self, runtime):
+        subjectdir = os.path.abspath(self.inputs.out_dir)
+        out_dir = subjectdir + '.bedpostX'
+
+        if isdefined(self.inputs.force_dir) and self.inputs.force_dir:
+            out_dir = os.path.abspath(self.inputs.out_dir)
+        self._out_dir = out_dir
+
+        if not os.path.exists(out_dir):
+            os.makedirs(out_dir)
+
+        _, _, ext = split_filename(self.inputs.mask)
+        copyfile(self.inputs.mask,
+                 os.path.join(subjectdir,
+                              'nodif_brain_mask' + ext))
+        _, _, ext = split_filename(self.inputs.dwi)
+        copyfile(self.inputs.dwi,
+                 os.path.join(subjectdir, 'data' + ext))
+        copyfile(self.inputs.bvals,
+                 os.path.join(subjectdir, 'bvals'))
+        copyfile(self.inputs.bvecs,
+                 os.path.join(subjectdir, 'bvecs'))
+
+        return super(BEDPOSTX5, self)._run_interface(runtime)
+
+    def _list_outputs(self):
+        outputs = self.output_spec().get()
+        out_dir = self._out_dir
+
+        post_fields = ['merged_thsamples', 'merged_phsamples',
+                       'merged_fsamples', 'dyads_disp']
+        for k in post_fields:
+            outputs[k] = []
+
+        for i in xrange(1, self.inputs.n_fibres + 1):
+            outputs['merged_thsamples'].append(self._gen_fname(('merged_th%d'
+                                               'samples') % i,
+                                               cwd=out_dir))
+            outputs['merged_phsamples'].append(self._gen_fname(('merged_ph%d'
+                                               'samples') % i,
+                                               cwd=out_dir))
+            outputs['merged_fsamples'].append(self._gen_fname(('merged_f%d'
+                                              'samples') % i,
+                                              cwd=out_dir))
+            outputs['mean_thsamples'].append(self._gen_fname(('mean_th%d'
+                                             'samples') % i,
+                                             cwd=out_dir))
+            outputs['mean_phsamples'].append(self._gen_fname(('mean_ph%d'
+                                             'samples') % i,
+                                             cwd=out_dir))
+            outputs['dyads_disp'].append(self._gen_fname(('dyads%d'
+                                         '_dispersion') % i, cwd=out_dir))
+
+        super_out = super(BEDPOSTX5, self)._list_outputs()
+
+        for k, v in super_out.iteritems():
+            outputs[k] = v
+
+        return outputs
+
+
+class XFibres5InputSpec(FSLXCommandInputSpec):
+    gradnonlin = File(exists=True, argstr='--gradnonlin=%s',
+                      desc='gradient file corresponding to slice')
+
+
+class XFibres5(FSLXCommand):
+    """
+    Perform model parameters estimation for local (voxelwise) diffusion
+    parameters
+    """
+    _cmd = 'xfibres'
+    input_spec = XFibres5InputSpec
+    output_spec = FSLXCommandOutputSpec
+
+
+class XFibres4InputSpec(FSLCommandInputSpec):
+    dwi = File(exists=True, argstr="--data=%s", mandatory=True)
+    mask = File(exists=True, argstr="--mask=%s", mandatory=True)
+    gradnonlin = File(exists=True, argstr="--gradnonlin=%s")
+    bvecs = File(exists=True, argstr="--bvecs=%s", mandatory=True)
+    bvals = File(exists=True, argstr="--bvals=%s", mandatory=True)
+    logdir = Directory("logdir", argstr="--logdir=%s", usedefault=True)
+    n_fibres = traits.Range(low=1, argstr="--nfibres=%d",
+                            desc="Maximum nukmber of fibres to fit in each voxel")
+    fudge = traits.Int(argstr="--fudge=%d",
+                         desc="ARD fudge factor")
+    n_jumps = traits.Range(low=1, argstr="--njumps=%d",
+                           desc="Num of jumps to be made by MCMC")
+    burn_in = traits.Range(low=0, argstr="--burnin=%d",
+                           desc="Total num of jumps at start of MCMC to be discarded")
+    burn_in_no_ard = traits.Range(low=0, argstr="--burninnoard=%d",
+                           desc="num of burnin jumps before the ard is imposed")
+    sample_every = traits.Range(low=0, argstr="--sampleevery=%d",
+                           desc="Num of jumps for each sample (MCMC)")
+    update_proposal_every = traits.Range(low=1, argstr="--updateproposalevery=%d",
+                           desc="Num of jumps for each update to the proposal density std (MCMC)")
+    seed = traits.Int(argstr="--seed=%d", desc="seed for pseudo random number generator")
+    model = traits.Int(argstr="--model=%d", desc="Which model to use. \
+1=mono-exponential (default and required for single shell). 2=continous \
+exponential (for multi-shell experiments)")
+
+    _xor_inputs1 = ('no_ard', 'all_ard')
+    no_ard = traits.Bool(argstr="--noard", desc="Turn ARD off on all fibres", xor=_xor_inputs1)
+    all_ard = traits.Bool(argstr="--allard", desc="Turn ARD on on all fibres", xor=_xor_inputs1)
+
+    _xor_inputs2 = ('no_spat', 'non_linear')
+    no_spat = traits.Bool(argstr="--nospat", desc="Initialise with tensor, not spatially", xor=_xor_inputs2)
+    non_linear = traits.Bool(argstr="--nonlinear", desc="Initialise with nonlinear fitting", xor=_xor_inputs2)
+    force_dir = traits.Bool(True,
+                               desc='use the actual directory name given - i.e. ' +
+                                    'do not add + to make a new directory',
+                               argstr='--forcedir', usedefault=True)
+
+
+class XFibres4OutputSpec(TraitedSpec):
+    dyads = OutputMultiPath(File(exists=True), desc="Mean of PDD distribution in vector form.")
+    fsamples = OutputMultiPath(File(exists=True), desc="Samples from the distribution on anisotropic volume fraction")
+    mean_dsamples = File(exists=True, desc="Mean of distribution on diffusivity d")
+    mean_fsamples = OutputMultiPath(File(exists=True), desc="Mean of distribution on f anisotropy")
+    mean_S0samples = File(exists=True, desc="Samples from S0 distribution")
+    phsamples = OutputMultiPath(File(exists=True), desc="Samples from the distribution on phi")
+    thsamples = OutputMultiPath(File(exists=True), desc="Samples from the distribution on theta")
+
+
+class XFibres4(FSLCommand):
+    """
+    Perform model parameters estimation for local (voxelwise) diffusion
+    parameters
+
+    .. deprecated:: 0.9.2
+      Use :class:`.XFibres5` instead.
+
+
+    """
+    _cmd = "xfibres"
+    input_spec = XFibres4InputSpec
+    output_spec = XFibres4OutputSpec
+
+    def __init__(self, **inputs):
+        warnings.warn(('Deprecated: Please use XFIBERS5 instead. This '
+                       'interface will be removed in version 0.11.'),
+                      DeprecationWarning)
+        super(XFibres4, self).__init__(**inputs)
+
+    def _list_outputs(self):
+        outputs = self.output_spec().get()
+        outputs["mean_dsamples"] = self._gen_fname("mean_dsamples",
+                                                   cwd=self.inputs.logdir)
+        outputs["mean_S0samples"] = self._gen_fname("mean_S0samples",
+                                                    cwd=self.inputs.logdir)
+        outputs["dyads"] = []
+        outputs["fsamples"] = []
+        outputs["mean_fsamples"] = []
+        outputs["phsamples"] = []
+        outputs["thsamples"] = []
+        for i in range(1, self.inputs.n_fibres + 1):
+            outputs["dyads"].append(self._gen_fname("dyads%d" % i,
+                                                    cwd=self.inputs.logdir))
+            outputs["fsamples"].append(self._gen_fname("f%dsamples" % i,
+                                                       cwd=self.inputs.logdir))
+            outputs["mean_fsamples"].append(self._gen_fname("mean_f%dsamples" % i,
+                                                            cwd=self.inputs.logdir))
+            outputs["phsamples"].append(self._gen_fname("ph%dsamples" % i,
+                                                        cwd=self.inputs.logdir))
+            outputs["thsamples"].append(self._gen_fname("th%dsamples" % i,
+                                                        cwd=self.inputs.logdir))
+
+        return outputs
+
+
+class BEDPOSTX4InputSpec(XFibres4InputSpec):
+    dwi = File(exists=True, desc='diffusion weighted image data file',
+               mandatory=True)
     mask = File(exists=True, desc='bet binary mask file', mandatory=True)
     bvecs = File(exists=True, desc='b vectors file', mandatory=True)
     bvals = File(exists=True, desc='b values file', mandatory=True)
     bpx_directory = Directory('bedpostx', argstr='%s', usedefault=True,
-                             desc='the name for this subject''s bedpostx folder')
-
-    fibres = traits.Int(1, argstr='-n %d', desc='number of fibres per voxel')
+                              desc=('the name for this subject\'s bedpostx'
+                                    ' folder'))
+    fibres = traits.Int(2, argstr='-n %d', desc='number of fibres per voxel')
     weight = traits.Float(1.00, argstr='-w %.2f',
-                          desc='ARD weight, more weight means less' +
-                               ' secondary fibres per voxel')
+                          desc=('ARD weight, more weight means less'
+                                ' secondary fibres per voxel'))
     burn_period = traits.Int(1000, argstr='-b %d', desc='burnin period')
     jumps = traits.Int(1250, argstr='-j %d', desc='number of jumps')
     sampling = traits.Int(25, argstr='-s %d', desc='sample every')
+    model = traits.Enum(1, 2, argstr='-model %d',
+                        desc=('model choice: monoexponential (1) or '
+                              'multiexponential (2). '))
+    nlgradient = traits.Bool(False, argstr='-g', desc=('consider gradient'
+                             'nonlinearities, default off'))
+    no_cuda = traits.Bool(False, argstr='-c',
+                          desc=('do not use CUDA capable hardware/queue '
+                                '(if found)'))
 
 
-class BEDPOSTXOutputSpec(TraitedSpec):
+class BEDPOSTX4OutputSpec(TraitedSpec):
     bpx_out_directory = Directory(exists=True,
                                   desc='path/name of directory with all ' +
                                        'bedpostx output files for this subject')
@@ -144,34 +530,40 @@ class BEDPOSTXOutputSpec(TraitedSpec):
     dyads = traits.List(File(exists=True),  desc='a list of path/name of mean of PDD distribution in vector form')
 
 
-class BEDPOSTX(FSLCommand):
-    """ Deprecated! Please use create_bedpostx_pipeline instead
+class BEDPOSTX4(FSLCommand):
+    """
+    bedpostx has an old interface, implemented here
+
 
     Example
     -------
 
     >>> from nipype.interfaces import fsl
-    >>> bedp = fsl.BEDPOSTX(bpx_directory='subjdir', bvecs='bvecs', bvals='bvals', dwi='diffusion.nii', \
-    mask='mask.nii', fibres=1)
+    >>> bedp = fsl.BEDPOSTX4(bpx_directory='subjdir', bvecs='bvecs', \
+bvals='bvals', dwi='diffusion.nii', mask='mask.nii', fibres=1)
     >>> bedp.cmdline
-    'bedpostx subjdir -n 1'
+    'bedpostx subjdir -n 1 --forcedir --logdir=logdir'
 
     """
 
     _cmd = 'bedpostx'
-    input_spec = BEDPOSTXInputSpec
-    output_spec = BEDPOSTXOutputSpec
+    input_spec = BEDPOSTX4InputSpec
+    output_spec = BEDPOSTX4OutputSpec
     _can_resume = True
 
     def __init__(self, **inputs):
-        warnings.warn("Deprecated: Please use create_bedpostx_pipeline instead", DeprecationWarning)
-        return super(BEDPOSTX, self).__init__(**inputs)
+        warnings.warn(('Deprecated: Please use BEDPOSTX5 or '
+                       'create_bedpostx_pipeline instead. This interface will '
+                       'be removed in version 0.11.'), DeprecationWarning)
+        super(BEDPOSTX4, self).__init__(**inputs)
 
-    def _run_interface(self, runtime):
+    def _get_bedpostx_dir(self):
+        return os.path.join(os.getcwd(), self.inputs.bpx_directory)
+
+    def _run_interface(self, runtime, correct_return_codes=[0]):
 
         #create the subject specific bpx_directory
-        bpx_directory = os.path.join(os.getcwd(), self.inputs.bpx_directory)
-        self.inputs.bpx_directory = bpx_directory
+        bpx_directory = self._get_bedpostx_dir()
         if not os.path.exists(bpx_directory):
             os.makedirs(bpx_directory)
 
@@ -187,18 +579,18 @@ class BEDPOSTX(FSLCommand):
             shutil.copyfile(self.inputs.bvecs,
                             os.path.join(self.inputs.bpx_directory, 'bvecs'))
 
-        runtime = super(BEDPOSTX, self)._run_interface(runtime)
+        runtime = super(BEDPOSTX4, self)._run_interface(runtime,
+                                                        correct_return_codes)
         if runtime.stderr:
             self.raise_exception(runtime)
         return runtime
 
     def _list_outputs(self):
         outputs = self.output_spec().get()
-        outputs['bpx_out_directory'] = os.path.join(os.getcwd(),
-                                    self.inputs.bpx_directory + '.bedpostX')
-        outputs['xfms_directory'] = os.path.join(os.getcwd(),
-                                    self.inputs.bpx_directory + '.bedpostX',
-                                    'xfms')
+        bpx_directory = self._get_bedpostx_dir()
+        outputs['bpx_out_directory'] = os.path.join(bpx_directory + '.bedpostX')
+        outputs['xfms_directory'] = os.path.join(bpx_directory + '.bedpostX',
+                                                 'xfms')
 
         for k in outputs.keys():
             if k not in ('outputtype', 'environ', 'args', 'bpx_out_directory',
@@ -230,7 +622,24 @@ class BEDPOSTX(FSLCommand):
         return outputs
 
 
-class ProbTrackXInputSpec(FSLCommandInputSpec):
+if (Info.version() and
+   LooseVersion(Info.version()) >= LooseVersion('5.0.0')):
+    CurrentXFibres = XFibres5
+    CurrentBEDPOST = BEDPOSTX5
+else:
+    CurrentXFibres = XFibres4
+    CurrentBEDPOST = BEDPOSTX4
+
+
+class XFibres(CurrentXFibres):
+    pass
+
+
+class BEDPOSTX(CurrentBEDPOST):
+    pass
+
+
+class ProbTrackXBaseInputSpec(FSLCommandInputSpec):
     thsamples = InputMultiPath(File(exists=True), mandatory=True)
     phsamples = InputMultiPath(File(exists=True), mandatory=True)
     fsamples = InputMultiPath(File(exists=True), mandatory=True)
@@ -243,21 +652,13 @@ class ProbTrackXInputSpec(FSLCommandInputSpec):
                          desc='seed volume(s), or voxel(s)' +
                          'or freesurfer label file',
                          argstr='--seed=%s', mandatory=True)
-    mode = traits.Enum("simple", "two_mask_symm", "seedmask",
-                       desc='options: simple (single seed voxel), seedmask (mask of seed voxels), '
-                            + 'twomask_symm (two bet binary masks) ',
-                       argstr='--mode=%s', genfile=True)
     target_masks = InputMultiPath(File(exits=True), desc='list of target masks - ' +
                        'required for seeds_to_targets classification', argstr='--targetmasks=%s')
-    mask2 = File(exists=True, desc='second bet binary mask (in diffusion space) in twomask_symm mode',
-                 argstr='--mask2=%s')
     waypoints = File(exists=True, desc='waypoint mask or ascii list of waypoint masks - ' +
                      'only keep paths going through ALL the masks', argstr='--waypoints=%s')
     network = traits.Bool(desc='activate network mode - only keep paths going through ' +
                           'at least one seed mask (required if multiple seed masks)',
                           argstr='--network')
-    mesh = File(exists=True, desc='Freesurfer-type surface descriptor (in ascii format)',
-                argstr='--mesh=%s')
     seed_ref = File(exists=True, desc='reference vol to define seed space in ' +
                    'simple mode - diffusion space assumed if absent',
                    argstr='--seedref=%s')
@@ -308,6 +709,17 @@ class ProbTrackXInputSpec(FSLCommandInputSpec):
                           argstr="--verbose=%d")
 
 
+class ProbTrackXInputSpec(ProbTrackXBaseInputSpec):
+    mode = traits.Enum("simple", "two_mask_symm", "seedmask",
+                       desc='options: simple (single seed voxel), seedmask (mask of seed voxels), '
+                            + 'twomask_symm (two bet binary masks) ',
+                       argstr='--mode=%s', genfile=True)
+    mask2 = File(exists=True, desc='second bet binary mask (in diffusion space) in twomask_symm mode',
+                 argstr='--mask2=%s')
+    mesh = File(exists=True, desc='Freesurfer-type surface descriptor (in ascii format)',
+                argstr='--mesh=%s')
+
+
 class ProbTrackXOutputSpec(TraitedSpec):
     log = File(exists=True, desc='path/name of a text record of the command that was run')
     fdt_paths = OutputMultiPath(File(exists=True), desc='path/name of a 3D image file containing the output ' +
@@ -436,6 +848,97 @@ class ProbTrackX(FSLCommand):
                 return "seedmask"
 
 
+class ProbTrackX2InputSpec(ProbTrackXBaseInputSpec):
+    simple = traits.Bool(desc='rack from a list of voxels (seed must be a ASCII list of coordinates)',
+                         usedefault=False, argstr='--simple')
+    fopd = File(exists=True, desc='Other mask for binning tract distribution',
+                argstr='--fopd=%s')
+    waycond = traits.Enum("OR", "AND", argstr='--waycond=%s',
+                          desc='Waypoint condition. Either "AND" (default) or "OR"')
+    wayorder = traits.Bool(desc='Reject streamlines that do not hit waypoints in given order. ' +
+                            'Only valid if waycond=AND', argstr='--wayorder')
+    onewaycondition = traits.Bool(desc='Apply waypoint conditions to each half tract separately',
+                                  argstr='--onewaycondition')
+    omatrix1 = traits.Bool(desc='Output matrix1 - SeedToSeed Connectivity',
+                           argstr='--omatrix1')
+    distthresh1 = traits.Float(argstr='--distthresh1=%.3f',
+                               desc='Discards samples (in matrix1) shorter than this threshold ' +
+                                '(in mm - default=0)')
+    omatrix2 = traits.Bool(desc='Output matrix2 - SeedToLowResMask', argstr='--omatrix2', requires=['target2'])
+    target2 = File(exists=True, desc='Low resolution binary brain mask for storing ' +
+                    'connectivity distribution in matrix2 mode', argstr='--target2=%s')
+    omatrix3 = traits.Bool(desc='Output matrix3 (NxN connectivity matrix)', argstr='--omatrix3',
+                           requires=['target3', 'lrtarget3'])
+    target3 = File(exists=True, desc='Mask used for NxN connectivity matrix ' +
+                    '(or Nxn if lrtarget3 is set)', argstr='--target3=%s')
+    lrtarget3 = File(exists=True, desc='Column-space mask used for Nxn connectivity matrix',
+                     argstr='--lrtarget3=%s')
+    distthresh3 = traits.Float(argstr='--distthresh3=%.3f',
+                               desc='Discards samples (in matrix3) shorter than this threshold ' +
+                                '(in mm - default=0)')
+    omatrix4 = traits.Bool(desc='Output matrix4 - DtiMaskToSeed (special Oxford Sparse Format)',
+                           argstr='--omatrix4')
+    colmask4 = File(exists=True, desc='Mask for columns of matrix4 (default=seed mask)',
+                     argstr='--colmask4=%s')
+    target4 = File(exists=True, desc='Brain mask in DTI space', argstr='--target4=%s')
+    meshspace = traits.Enum("caret", "freesurfer", "first", "vox", argstr='--meshspace=%s',
+                            desc='Mesh reference space - either "caret" (default) or ' +
+                             '"freesurfer" or "first" or "vox"')
+
+
+class ProbTrackX2OutputSpec(ProbTrackXOutputSpec):
+    network_matrix = File(exists=True, desc='the network matrix generated by --omatrix1 option')
+    matrix1_dot = File(exists=True, desc='Output matrix1.dot - SeedToSeed Connectivity')
+    lookup_tractspace = File(exists=True, desc='lookup_tractspace generated by --omatrix2 option')
+    matrix2_dot = File(exists=True, desc='Output matrix2.dot - SeedToLowResMask')
+    matrix3_dot = File(exists=True, desc='Output matrix3 - NxN connectivity matrix')
+
+
+class ProbTrackX2(ProbTrackX):
+    """ Use FSL  probtrackx2 for tractography on bedpostx results
+
+    Examples
+    --------
+
+    >>> from nipype.interfaces import fsl
+    >>> pbx2 = fsl.ProbTrackX2()
+    >>> pbx2.inputs.seed = 'seed_source.nii.gz'
+    >>> pbx2.inputs.thsamples = 'merged_th1samples.nii.gz'
+    >>> pbx2.inputs.fsamples = 'merged_f1samples.nii.gz'
+    >>> pbx2.inputs.phsamples = 'merged_ph1samples.nii.gz'
+    >>> pbx2.inputs.mask = 'nodif_brain_mask.nii.gz'
+    >>> pbx2.inputs.out_dir = '.'
+    >>> pbx2.inputs.n_samples = 3
+    >>> pbx2.inputs.n_steps = 10
+    >>> pbx2.cmdline
+    'probtrackx2 --forcedir -m nodif_brain_mask.nii.gz --nsamples=3 --nsteps=10 --opd --dir=. --samples=merged --seed=seed_source.nii.gz'
+    """
+    _cmd = 'probtrackx2'
+    input_spec = ProbTrackX2InputSpec
+    output_spec = ProbTrackX2OutputSpec
+
+    def _list_outputs(self):
+        outputs = super(ProbTrackX2, self)._list_outputs()
+
+        if not isdefined(self.inputs.out_dir):
+            out_dir = os.getcwd()
+        else:
+            out_dir = self.inputs.out_dir
+
+        if isdefined(self.inputs.omatrix1):
+            outputs['network_matrix'] = os.path.abspath(os.path.join(out_dir, 'fdt_network_matrix'))
+            outputs['matrix1_dot'] = os.path.abspath(os.path.join(out_dir, 'fdt_matrix1.dot'))
+
+        if isdefined(self.inputs.omatrix2):
+            outputs['lookup_tractspace'] = \
+              os.path.abspath(os.path.join(out_dir, 'lookup_tractspace_fdt_matrix2.nii.gz'))
+            outputs['matrix2_dot'] = os.path.abspath(os.path.join(out_dir, 'fdt_matrix2.dot'))
+
+        if isdefined(self.inputs.omatrix3):
+            outputs['matrix3_dot'] = os.path.abspath(os.path.join(out_dir, 'fdt_matrix3.dot'))
+        return outputs
+
+
 class VecRegInputSpec(FSLCommandInputSpec):
     in_file = File(exists=True, argstr='-i %s', desc='filename for input vector or tensor field',
                   mandatory=True)
@@ -773,89 +1276,6 @@ class DistanceMap(FSLCommand):
         return None
 
 
-class XFibresInputSpec(FSLCommandInputSpec):
-    dwi = File(exists=True, argstr="--data=%s", mandatory=True)
-    mask = File(exists=True, argstr="--mask=%s", mandatory=True)
-    gradnonlin = File(exists=True, argstr="--gradnonlin=%s")
-    bvecs = File(exists=True, argstr="--bvecs=%s", mandatory=True)
-    bvals = File(exists=True, argstr="--bvals=%s", mandatory=True)
-    logdir = Directory("logdir", argstr="--logdir=%s", usedefault=True)
-    n_fibres = traits.Range(low=1, argstr="--nfibres=%d",
-                            desc="Maximum nukmber of fibres to fit in each voxel")
-    fudge = traits.Int(argstr="--fudge=%d",
-                         desc="ARD fudge factor")
-    n_jumps = traits.Range(low=1, argstr="--njumps=%d",
-                           desc="Num of jumps to be made by MCMC")
-    burn_in = traits.Range(low=0, argstr="--burnin=%d",
-                           desc="Total num of jumps at start of MCMC to be discarded")
-    burn_in_no_ard = traits.Range(low=0, argstr="--burninnoard=%d",
-                           desc="num of burnin jumps before the ard is imposed")
-    sample_every = traits.Range(low=0, argstr="--sampleevery=%d",
-                           desc="Num of jumps for each sample (MCMC)")
-    update_proposal_every = traits.Range(low=1, argstr="--updateproposalevery=%d",
-                           desc="Num of jumps for each update to the proposal density std (MCMC)")
-    seed = traits.Int(argstr="--seed=%d", desc="seed for pseudo random number generator")
-    model = traits.Int(argstr="--model=%d", desc="Which model to use. \
-1=mono-exponential (default and required for single shell). 2=continous \
-exponential (for multi-shell experiments)")
-
-    _xor_inputs1 = ('no_ard', 'all_ard')
-    no_ard = traits.Bool(argstr="--noard", desc="Turn ARD off on all fibres", xor=_xor_inputs1)
-    all_ard = traits.Bool(argstr="--allard", desc="Turn ARD on on all fibres", xor=_xor_inputs1)
-
-    _xor_inputs2 = ('no_spat', 'non_linear')
-    no_spat = traits.Bool(argstr="--nospat", desc="Initialise with tensor, not spatially", xor=_xor_inputs2)
-    non_linear = traits.Bool(argstr="--nonlinear", desc="Initialise with nonlinear fitting", xor=_xor_inputs2)
-    force_dir = traits.Bool(True,
-                               desc='use the actual directory name given - i.e. ' +
-                                    'do not add + to make a new directory',
-                               argstr='--forcedir', usedefault=True)
-
-
-class XFibresOutputSpec(TraitedSpec):
-    dyads = OutputMultiPath(File(exists=True), desc="Mean of PDD distribution in vector form.")
-    fsamples = OutputMultiPath(File(exists=True), desc="Samples from the distribution on anisotropic volume fraction")
-    mean_dsamples = File(exists=True, desc="Mean of distribution on diffusivity d")
-    mean_fsamples = OutputMultiPath(File(exists=True), desc="Mean of distribution on f anisotropy")
-    mean_S0samples = File(exists=True, desc="Samples from S0 distribution")
-    phsamples = OutputMultiPath(File(exists=True), desc="Samples from the distribution on phi")
-    thsamples = OutputMultiPath(File(exists=True), desc="Samples from the distribution on theta")
-
-
-class XFibres(FSLCommand):
-    """Perform model parameters estimation for local (voxelwise) diffusion parameters
-    """
-    _cmd = "xfibres"
-    input_spec = XFibresInputSpec
-    output_spec = XFibresOutputSpec
-
-    def _run_interface(self, runtime):
-        runtime = super(XFibres, self)._run_interface(runtime)
-        if runtime.stderr:
-            self.raise_exception(runtime)
-        return runtime
-
-    def _list_outputs(self):
-        outputs = self.output_spec().get()
-        outputs["mean_dsamples"] = self._gen_fname("mean_dsamples",
-                                                   cwd=self.inputs.logdir)
-        outputs["mean_S0samples"] = self._gen_fname("mean_S0samples",
-                                                    cwd=self.inputs.logdir)
-        outputs["dyads"] = []
-        outputs["fsamples"] = []
-        outputs["mean_fsamples"] = []
-        outputs["phsamples"] = []
-        outputs["thsamples"] = []
-        for i in range(1, self.inputs.n_fibres + 1):
-            outputs["dyads"].append(self._gen_fname("dyads%d" % i, cwd=self.inputs.logdir))
-            outputs["fsamples"].append(self._gen_fname("f%dsamples" % i, cwd=self.inputs.logdir))
-            outputs["mean_fsamples"].append(self._gen_fname("mean_f%dsamples" % i, cwd=self.inputs.logdir))
-            outputs["phsamples"].append(self._gen_fname("ph%dsamples" % i, cwd=self.inputs.logdir))
-            outputs["thsamples"].append(self._gen_fname("th%dsamples" % i, cwd=self.inputs.logdir))
-
-        return outputs
-
-
 class MakeDyadicVectorsInputSpec(FSLCommandInputSpec):
     theta_vol = File(exists=True, mandatory=True, position=0, argstr="%s")
     phi_vol = File(exists=True, mandatory=True, position=1, argstr="%s")
