@@ -22,7 +22,7 @@ import numpy as np
 from nipype.interfaces.base import (OutputMultiPath, TraitedSpec, isdefined,
                                     traits, InputMultiPath, File)
 from nipype.interfaces.spm.base import (SPMCommand, scans_for_fname,
-                                        func_is_3d,
+                                        func_is_3d, Info,
                                         scans_for_fnames, SPMCommandInputSpec)
 from nipype.utils.filemanip import (fname_presuffix, filename_to_list,
                                     list_to_filename, split_filename)
@@ -600,8 +600,12 @@ class Segment(SPMCommand):
 
     """
 
-    _jobtype = 'spatial'
-    _jobname = 'preproc'
+    if Info.version() and Info.version()['name'] == "SPM12":
+        _jobtype = 'tools'
+        _jobname = 'oldseg'
+    else:
+        _jobtype = 'spatial'
+        _jobname = 'preproc'
 
     input_spec = SegmentInputSpec
     output_spec = SegmentOutputSpec
@@ -666,7 +670,7 @@ class NewSegmentInputSpec(SPMCommandInputSpec):
             - tissue probability map (4D), 1-based index to frame
             - number of gaussians
             - which maps to save [Native, DARTEL] - a tuple of two boolean values
-            - which maps to save [Modulated, Unmodualted] - a tuple of two boolean values""",
+            - which maps to save [Unmodulated, Modulated] - a tuple of two boolean values""",
             field='tissue')
     affine_regularization = traits.Enum('mni', 'eastern', 'subj', 'none', field='warp.affreg',
                       desc='mni, eastern, subj, none ')
@@ -723,8 +727,12 @@ class NewSegment(SPMCommand):
 
     input_spec = NewSegmentInputSpec
     output_spec = NewSegmentOutputSpec
-    _jobtype = 'tools'
-    _jobname = 'preproc8'
+    if Info.version() and Info.version()['name'] == "SPM12":
+        _jobtype = 'spatial'
+        _jobname = 'preproc'
+    else:
+        _jobtype = 'tools'
+        _jobname = 'preproc8'
 
     def _format_arg(self, opt, spec, val):
         """Convert input to appropriate format for spm
@@ -1084,6 +1092,8 @@ class CreateWarpedInputSpec(SPMCommandInputSpec):
                               field='crt_warped.K')
     interp = traits.Range(low=0, high=7, field='crt_warped.interp',
                           desc='degree of b-spline used for interpolation')
+    modulate = traits.Bool(field='crt_warped.jactransf',
+                           desc="Modulate images")
 
 
 class CreateWarpedOutputSpec(TraitedSpec):
@@ -1127,8 +1137,12 @@ class CreateWarped(SPMCommand):
         outputs['warped_files'] = []
         for filename in self.inputs.image_files:
             pth, base, ext = split_filename(filename)
-            outputs['warped_files'].append(os.path.realpath('w%s%s' % (base,
-                                                                       ext)))
+            if isdefined(self.inputs.modulate) and self.inputs.modulate:
+                outputs['warped_files'].append(os.path.realpath('mw%s%s' % (base,
+                                                                            ext)))
+            else:
+                outputs['warped_files'].append(os.path.realpath('w%s%s' % (base,
+                                                                           ext)))
         return outputs
 
 
