@@ -11,7 +11,7 @@ import nipype.interfaces.base as nib
 from nipype.utils.filemanip import split_filename
 from nipype.interfaces.base import Undefined, config
 from traits.testing.nose_tools import skip
-
+import traits.api as traits
 #test Bunch
 def test_bunch():
     b = nib.Bunch()
@@ -181,11 +181,15 @@ def test_namesource():
         moo = nib.File(name_source=['doo'], hash_files=False, argstr="%s",
                        position=2)
         doo = nib.File(exists=True, argstr="%s", position=1)
+        goo = traits.Int(argstr="%d", position=4)
+        poo = nib.File(name_source=['goo'], hash_files=False, argstr="%s",position=3)
+
     class TestName(nib.CommandLine):
         _cmd = "mycommand"
         input_spec = spec2
     testobj = TestName()
     testobj.inputs.doo = tmp_infile
+    testobj.inputs.goo = 99
     yield assert_true, '%s_generated' % nme in testobj.cmdline
     testobj.inputs.moo = "my_%s_template"
     yield assert_true, 'my_%s_template' % nme in testobj.cmdline
