@@ -4,9 +4,10 @@
 """
 __docformat__ = 'restructuredtext'
 
-from nipype.utils.misc import package_check
+from ..utils.misc import package_check
 from nose import SkipTest
 
+
 def skip_if_no_package(*args, **kwargs):
     """Raise SkipTest if package_check fails
 
@@ -18,4 +19,3 @@ def skip_if_no_package(*args, **kwargs):
     package_check(exc_failed_import=SkipTest,
                   exc_failed_check=SkipTest,
                   *args, **kwargs)
-
