@@ -4,17 +4,22 @@ import nipype.pipeline.engine as pe
 import nipype.interfaces.utility as niu
 import nipype.interfaces.fsl as fsl
 import os
+import warnings
 
 def create_dmri_preprocessing(name='dMRI_preprocessing', use_fieldmap=True, fieldmap_registration=False):
-    """Creates a workflow that chains the necessary pipelines to
+    """
+    Creates a workflow that chains the necessary pipelines to
     correct for motion, eddy currents, and, if selected, susceptibility
     artifacts in EPI dMRI sequences.
 
-    .. warning::
+    .. deprecated:: 0.9.3
+      Use :func:`nipype.workflows.dmri.preprocess.epi.all_fmb_pipeline` or
+      :func:`nipype.workflows.dmri.preprocess.epi.all_peb_pipeline` instead.
 
-    IMPORTANT NOTICE: this workflow rotates the b-vectors, so please be adviced
-    that not all the dicom converters ensure the consistency between the resulting
-    nifti orientation and the b matrix table (e.g. dcm2nii checks it).
+
+    .. warning:: This workflow rotates the b-vectors, so please be
+      advised that not all the dicom converters ensure the consistency between the resulting
+      nifti orientation and the b matrix table (e.g. dcm2nii checks it).
 
 
     Example
@@ -54,12 +59,16 @@ def create_dmri_preprocessing(name='dMRI_preprocessing', use_fieldmap=True, fiel
 
 
     Optional arguments::
+
         use_fieldmap - True if there are fieldmap files that should be used (default True)
         fieldmap_registration - True if registration to fieldmap should be performed (default False)
 
 
     """
 
+    warnings.warn(('This workflow is deprecated from v.1.0.0, use of available '
+                  'nipype.workflows.dmri.preprocess.epi.all_*'), DeprecationWarning)
+
     pipeline = pe.Workflow(name=name)
 
     inputnode = pe.Node(niu.IdentityInterface(
@@ -118,11 +127,14 @@ def create_motion_correct_pipeline(name='motion_correct'):
     (Leemans et al. 2009 - http://www.ncbi.nlm.nih.gov/pubmed/19319973),
     making use of the rotation matrix obtained by FLIRT.
 
-    .. warning::
 
-    IMPORTANT NOTICE: this workflow rotates the b-vectors, so please be adviced
-    that not all the dicom converters ensure the consistency between the resulting
-    nifti orientation and the b matrix table (e.g. dcm2nii checks it).
+    .. deprecated:: 0.9.3
+      Use :func:`nipype.workflows.dmri.preprocess.epi.hmc_pipeline` instead.
+
+
+    .. warning:: This workflow rotates the b-vectors, so please be adviced
+      that not all the dicom converters ensure the consistency between the resulting
+      nifti orientation and the b matrix table (e.g. dcm2nii checks it).
 
 
     Example
@@ -147,6 +159,10 @@ def create_motion_correct_pipeline(name='motion_correct'):
 
     """
 
+    warnings.warn(('This workflow is deprecated from v.1.0.0, use '
+                  'nipype.workflows.dmri.preprocess.epi.hmc_pipeline instead'),
+                  DeprecationWarning)
+
     inputnode = pe.Node(
         niu.IdentityInterface(
             fields=['in_file', 'ref_num', 'in_bvec']),
@@ -183,7 +199,13 @@ def create_motion_correct_pipeline(name='motion_correct'):
 
 
 def create_eddy_correct_pipeline(name='eddy_correct'):
-    """Creates a pipeline that replaces eddy_correct script in FSL. It takes a
+    """
+
+    .. deprecated:: 0.9.3
+      Use :func:`nipype.workflows.dmri.preprocess.epi.ecc_pipeline` instead.
+
+
+    Creates a pipeline that replaces eddy_correct script in FSL. It takes a
     series of diffusion weighted images and linearly co-registers them to one
     reference image. No rotation of the B-matrix is performed, so this pipeline
     should be executed after the motion correction pipeline.
@@ -206,6 +228,10 @@ def create_eddy_correct_pipeline(name='eddy_correct'):
         outputnode.eddy_corrected
     """
 
+    warnings.warn(('This workflow is deprecated from v.1.0.0, use '
+                  'nipype.workflows.dmri.preprocess.epi.ecc_pipeline instead'),
+                  DeprecationWarning)
+
     inputnode = pe.Node(
         niu.IdentityInterface(fields=['in_file', 'ref_num']),
                         name='inputnode')
@@ -237,6 +263,11 @@ def create_eddy_correct_pipeline(name='eddy_correct'):
 
 def fieldmap_correction(name='fieldmap_correction', nocheck=False):
     """
+
+    .. deprecated:: 0.9.3
+      Use :func:`nipype.workflows.dmri.preprocess.epi.sdc_fmb` instead.
+
+
     Fieldmap-based retrospective correction of EPI images for the susceptibility distortion
     artifact (Jezzard et al., 1995). Fieldmap images are assumed to be already registered
     to EPI data, and a brain mask is required.
@@ -246,7 +277,6 @@ def fieldmap_correction(name='fieldmap_correction', nocheck=False):
     available as of FSL 5.0.
 
 
-
     Example
     -------
 
@@ -282,6 +312,10 @@ def fieldmap_correction(name='fieldmap_correction', nocheck=False):
 
     """
 
+    warnings.warn(('This workflow is deprecated from v.1.0.0, use '
+                  'nipype.workflows.dmri.preprocess.epi.sdc_fmb instead'),
+                  DeprecationWarning)
+
     inputnode = pe.Node(niu.IdentityInterface(
                         fields=['in_file',
                         'in_mask',
@@ -337,7 +371,7 @@ def fieldmap_correction(name='fieldmap_correction', nocheck=False):
                     ,(inputnode,      mask_mag, [('in_mask', 'mask_file' )])
                     ,(select_mag,     mask_mag, [('roi_file', 'in_file')])
                     ,(mask_mag,        fslprep, [('out_file', 'in_magnitude')])
-                    ,(fslprep,             vsm, [('out_fieldmap', 'phasemap_file')])
+                    ,(fslprep,             vsm, [('out_fieldmap', 'phasemap_in_file')])
                     ,(inputnode,           vsm, [('fieldmap_mag', 'in_file'),
                                                  ('encoding_direction','unwarp_direction'),
                                                  (('te_diff', _ms2sec), 'asym_se_time'),
@@ -360,8 +394,13 @@ def fieldmap_correction(name='fieldmap_correction', nocheck=False):
 
 def topup_correction( name='topup_correction' ):
     """
-        Corrects for susceptibilty distortion of EPI images when one reverse encoding dataset has
-        been acquired
+
+    .. deprecated:: 0.9.3
+      Use :func:`nipype.workflows.dmri.preprocess.epi.sdc_peb` instead.
+
+
+    Corrects for susceptibilty distortion of EPI images when one reverse encoding dataset has
+    been acquired
 
 
     Example
@@ -370,22 +409,30 @@ def topup_correction( name='topup_correction' ):
     >>> nipype_epicorrect = topup_correction('nipype_topup')
     >>> nipype_epicorrect.inputs.inputnode.in_file_dir = 'epi.nii'
     >>> nipype_epicorrect.inputs.inputnode.in_file_rev = 'epi_rev.nii'
-    >>> nipype_epicorrect.inputs.inputnode.encoding_direction = 'y'
+    >>> nipype_epicorrect.inputs.inputnode.encoding_direction = ['y', 'y-']
     >>> nipype_epicorrect.inputs.inputnode.ref_num = 0
     >>> nipype_epicorrect.run() # doctest: +SKIP
 
+
     Inputs::
+
         inputnode.in_file_dir - EPI volume acquired in 'forward' phase encoding
         inputnode.in_file_rev - EPI volume acquired in 'reversed' phase encoding
         inputnode.encoding_direction - Direction encoding of in_file_dir
         inputnode.ref_num - Identifier of the reference volumes (usually B0 volume)
 
+
     Outputs::
 
         outputnode.epi_corrected
 
 
     """
+
+    warnings.warn(('This workflow is deprecated from v.1.0.0, use '
+                  'nipype.workflows.dmri.preprocess.epi.sdc_peb instead'),
+                  DeprecationWarning)
+
     pipeline = pe.Workflow(name=name)
 
     inputnode = pe.Node(niu.IdentityInterface(
@@ -400,7 +447,6 @@ def topup_correction( name='topup_correction' ):
     outputnode = pe.Node( niu.IdentityInterface(
                           fields=['out_fieldcoef',
                                   'out_movpar',
-                                  'out_topup',
                                   'out_enc_file',
                                   'epi_corrected'
                           ]), name='outputnode'
@@ -424,10 +470,11 @@ def topup_correction( name='topup_correction' ):
                      ,(combin,        merged, [('out', 'in_files')] )
                      ,(merged,         topup, [('merged_file','in_file')])
                      ,(inputnode,      topup, [('encoding_direction','encoding_direction'),('readout_times','readout_times') ])
-                     ,(topup,     applytopup, [('out_topup','in_topup'),('out_enc_file','encoding_file')])
+                     ,(topup,     applytopup, [('out_fieldcoef','in_topup_fieldcoef'),('out_movpar','in_topup_movpar'),
+                                               ('out_enc_file','encoding_file')])
                      ,(combin2,   applytopup, [('out','in_files')] )
                      ,(topup,     outputnode, [('out_fieldcoef','out_fieldcoef'),('out_movpar','out_movpar'),
-                                               ('out_topup','out_topup'),('out_enc_file','out_enc_file') ])
+                                               ('out_enc_file','out_enc_file') ])
                      ,(applytopup,outputnode, [('out_corrected','epi_corrected')])
                      ])
 
@@ -435,11 +482,18 @@ def topup_correction( name='topup_correction' ):
 
 
 def create_epidewarp_pipeline(name='epidewarp', fieldmap_registration=False):
-    """ Replaces the epidewarp.fsl script (http://www.nmr.mgh.harvard.edu/~greve/fbirn/b0/epidewarp.fsl)
+    """
+    Replaces the epidewarp.fsl script (http://www.nmr.mgh.harvard.edu/~greve/fbirn/b0/epidewarp.fsl)
     for susceptibility distortion correction of dMRI & fMRI acquired with EPI sequences and the fieldmap
     information (Jezzard et al., 1995) using FSL's FUGUE. The registration to the (warped) fieldmap
     (strictly following the original script) is available using fieldmap_registration=True.
 
+
+    .. warning:: This workflow makes use of ``epidewarp.fsl`` a script of FSL deprecated long
+      time ago. The use of this workflow is not recommended, use
+      :func:`nipype.workflows.dmri.preprocess.epi.sdc_fmb` instead.
+
+
     Example
     -------
 
@@ -479,6 +533,9 @@ def create_epidewarp_pipeline(name='epidewarp', fieldmap_registration=False):
 
     """
 
+    warnings.warn(('This workflow reproduces a deprecated FSL script.'),
+                  DeprecationWarning)
+
     inputnode = pe.Node(niu.IdentityInterface(fields=['in_file',
                                                                    'fieldmap_mag',
                                                                    'fieldmap_pha',
@@ -550,7 +607,7 @@ def create_epidewarp_pipeline(name='epidewarp', fieldmap_registration=False):
                     ,(mask_mag_dil,    prelude, [('out_file', 'mask_file')])
                     ,(prelude,      fill_phase, [('unwrapped_phase_file', 'in_file')])
                     ,(inputnode,           vsm, [('fieldmap_mag', 'in_file')])
-                    ,(fill_phase,          vsm, [('out_file', 'phasemap_file')])
+                    ,(fill_phase,          vsm, [('out_file', 'phasemap_in_file')])
                     ,(inputnode,           vsm, [(('te_diff', _ms2sec), 'asym_se_time'), ('vsm_sigma', 'smooth2d')])
                     ,(dwell_time,          vsm, [(('dwell_time', _ms2sec), 'dwell_time')])
                     ,(mask_mag_dil,        vsm, [('out_file', 'mask_file')])
@@ -574,7 +631,7 @@ def create_epidewarp_pipeline(name='epidewarp', fieldmap_registration=False):
         # fugue -i %s -w %s --loadshift=%s --mask=%s % ( mag_name, magfw_name,
         # vsmmag_name, mask_name ), log ) # Forward Map
         vsm_fwd = pe.Node(fsl.FUGUE(
-            save_warped=True), name='vsm_fwd')
+            forward_warping=True), name='vsm_fwd')
         vsm_reg = pe.Node(fsl.FLIRT(bins=256, cost='corratio', dof=6, interp='spline',  searchr_x=[
                           -10, 10], searchr_y=[-10, 10], searchr_z=[-10, 10]), name='vsm_registration')
         # 'flirt -in %s -ref %s -out %s -init %s -applyxfm' % ( vsmmag_name, ref_epi, vsmmag_name, magfw_mat_out )
